@@ -44,11 +44,11 @@ def init():
 ### Simulation parameters
 
     simulation['name']='someMMpXRTsimulation' #Defines the output filename of results.
-    simulation['version']="1.8"
+    simulation['version']="1.10"
     simulation['comment']='someMMpXRTsimulation' #This is just written in the figure as subtitle for convinience.
     simulation['num_processes']=1 #Number of processors to be used by the simulation.
     simulation['out_data_directory']='./datafiles/' #Where the ouptupt data are stored.
-
+    simulation['DetectorXlim']=[]
     #How much rays should be calculated. The more the better, but longer.
     simulation['numraysB']=1e5 #Number of rays for the broadband simulation.
     simulation['numraysM']=1e5 #Number of rays for the monocrhomatic simulation.
@@ -63,12 +63,16 @@ def init():
     simulation['PSFStepY']=-1
     simulation['PSFStepX']=-1
 
+    simulation['non_central_PSFs']=0 #instead of broadband simulation, I will simulate a bunnch of PSFs
+
     #numerical parameters, default values should be ok for most cases.
     #When details of mossaic stuff at non-optimal angles are important, having higher values makes more precision, like e.g. width=30, numpoints=50 was used for mossaicity section of the CPC paper
     simulation['numerical_mossaic_phimap_width']=10
     simulation['numerical_mossaic_phimap_numpoints']=14
     simulation['numerical_intersect_halving']=0 #selects whcih algorithm is used for torus_line_intersect; 0-the standard, pretty fast; 1-'halving of interval': more robust, slower
 
+    simulation['DetectorStepY']=-1
+    simulation['DetectorStepX']=-1
 ### Crystal
     ## General
     crystal['d2']=4.00 #crystal spacing 2d [Angstrom]
@@ -140,8 +144,7 @@ def init():
     source['rOffset']=[0 ,0, 0] #offset of part of the beam given by previous value
     source['continuum']=False
     source['continuumMarks']=False#add a marks with
-    source['continuumMarksSpread']=200
-    source['continuumMarksCount']=3
+    source['continuumMarks_array']=[]
     source['continuumAdd']=False#add a little bit of continuum to the simuled x-ray spectrum
     source['continuumAddedRatio']=0.3#fraction of rays that will have that added continuum
     source['useBeamSpectrum']=False
@@ -189,7 +192,6 @@ def load(p):
     name=p['simulation']['name']
     rrrs=pickle.load(open( dira+ "mmpxrt_results_"+name+"", "rb" ) )
     p=  pickle.load(open( dira+ "mmpxrt_parameters_"+name, "rb" ) )
-    #print (p['crystal'])
     return rrrs,p
 
 
@@ -485,7 +487,7 @@ def sphere_ray_intersect(sphere_centre, sphere_radius, ray_start,ray_direction):
 
 def torus_ray_intersectElementwise(crystalRvSq,crystalRSq,crystalRv,crystalR,crystalCenterX,deepinCrystal, ray_start,ray_direction,crystalLength):
 ## FIND INTERSECTION OF RAY WITH CRYSTAL PLANE  (hlaving the interval, each call took 44ms)
-    #first, find "tF" : 't' of intersection with horizontal plane
+    ##first, find "tF" : 't' of intersection with horizontal plane
     r0=ray_start
     v0=ray_direction
     tF=-1.*r0[2]/v0[2];
@@ -541,8 +543,7 @@ def torus_ray_intersect(crystalRvSq,crystalRSq,crystalRv,crystalR,crystalCenterX
     v02=v0[2]
     t=tF
     maxIter=30
-    precision=1e-5
-    precision=1e-3
+    precision=1e-5 #this means precision of 10μm, seems ok for tested applications
     i=0
     maxZ=100#maximal 'height' of crystal from floor. hard limit
     while (True):
@@ -555,14 +556,11 @@ def torus_ray_intersect(crystalRvSq,crystalRSq,crystalRv,crystalR,crystalCenterX
            print(i)
            print("X {:.0f}, Y {:.1f}, Z {:.0f}, t={:.0f}  ".format(R0,R1,R2,t))
        sqrt=(crystalRvSq-((R0-crystalCenterX)*(R0-crystalCenterX)))
-#       print(sqrt)
        if sqrt<0:
            parzH=maxZ #too far from the crystal
        else:
            parzH=-1* math.sqrt(sqrt)  + crystalRv #how high is the crystal above the floor based on the radius along length
-       #print("parzH {:.0f}".format(parzH))
        if crystalRSq-(R1*R1)<0:#the intersection would be too far,
-#           print("i= {:2.0f}, t={:.2f} ".format(i,t))
            if debug:
                print('step back, t={:.0f}, R1={:.0f}'.format(t,R1))
            t=0.8*t
@@ -661,7 +659,6 @@ def spectrometer(p):
         spectrorrr : Dictionary of rrr of both runs.
     """
     p['source']['continuumAdd']=True
-    p['source']['continuumMarks']=False
     p['source']['continuumAddedRatio']=0.
 
     if p['source']['divergenceFWHM']<0:
@@ -688,7 +685,7 @@ def spectrometer(p):
     print( 'Going to raytrace (broadband)')
     s['continuum']=True
     sg['numrays']=p['simulation']['numraysB']
-    s['rOffset']=[0,0,0]
+    #s['rOffset']=[0,0,0]leave rOffset to be set by user
     sg['windowX']=10
     sg['windowY']=10
 
@@ -703,6 +700,7 @@ def spectrometer(p):
     ## Second run: Monochromatic
     print( '\nGoing to raytrace (monochromatic)')
     s['continuum']=False
+    p['source']['continuumMarks']=False
     s['secondBeamRatio']=0
     s['showrealspatial']=True
     s['rOffsetRatio']=0.2
@@ -731,6 +729,85 @@ def spectrometer(p):
     return spectrorrr
 
 
+def spectrometer_imager(p):
+    """ This function makes the two raytracing tests of cyrstal spectrometer.
+
+    Params:
+        p : Dictionary of dictionaries of parameters.
+
+    Returns:
+        spectrorrr : Dictionary of rrr of both runs.
+    """
+    p['source']['continuumAdd']=True
+    p['source']['continuumMarks']=False
+    p['source']['continuumAddedRatio']=0.
+
+    if p['source']['divergenceFWHM']<0:
+        p['source']['divergenceAutomatic']=True
+    p['simulation']['collectLostBeams']=False
+
+    if p['simulation']['numraysE']>0:
+        p['simulation']['numraysB']=round(10**(p['simulation']['numraysE']))
+        p['simulation']['numraysM']=round(10**(p['simulation']['numraysE']-1))
+
+    windowY=30
+    p=geometry(p)
+    sim=p['simulation']
+    s=p['source']
+    sg=p['sg']
+    nump=sim['num_processes'];
+
+
+    ## First run: broadband
+    start = time.time()
+    print( 'MMPXRT spectrometer ({:2.0f} cores)'.format(nump))
+
+    print( 'Going to raytrace (broadband)')
+    s['continuum']=True
+    sg['numrays']=p['simulation']['numraysB']
+    sg['windowX']=10
+    sg['windowY']=10
+    s['showrealspatial']=0
+    s['show2Dspatial']=True
+
+    if nump>1:
+        sg['numrays']=int(np.round(sg['numrays']/nump))
+        broadrrr = raytrace_multiprocess(p,nump)
+    else:
+        broadrrr = raytrace(p,None,None)
+
+    end = time.time()
+
+    ## Second run: Monochromatic
+    print( '\nGoing to raytrace (monochromatic)')
+    s['continuum']=False
+    s['secondBeamRatio']=0
+    #s['showrealspatial']=True
+    s['show2Dspatial']=True
+    s['useBeamSpectrum']=0
+    sg['numrays']=p['simulation']['numraysM']
+    if nump>1:
+        sg['numrays']=int(np.round(sg['numrays']/nump))
+        monorrr = raytrace_multiprocess(p,nump)
+    else:
+        monorrr = raytrace(p,None,None)
+
+    print("\n\n Spectrometer took: "+str(np.round(end-start))+" s");
+
+    spectrorrr={}
+    spectrorrr['elapsedTime']=end-start
+    spectrorrr['mono']=monorrr
+    spectrorrr['broad']=broadrrr
+
+    fn=p['simulation']['out_data_directory']  +'mmpxrt_results_' +p['simulation']['name']
+    pickle.dump( spectrorrr, open( fn, "wb" ) )
+    fn=p['simulation']['out_data_directory']  +'mmpxrt_parameters_' +p['simulation']['name']
+    pickle.dump( p, open( fn, "wb" ) )
+
+    return spectrorrr
+
+
+
 ###############################################
 def geometry(p):
     g=p['geometry']
@@ -919,7 +996,7 @@ def raytrace(p,q,progress):
     spatialoffset=False
     totati=0
     totgol=0
-
+    rOffset=np.linalg.norm(p['source']['rOffset'])
     #finding the total optimal probability
     N=[0,0,1]
     L=[np.cos(ThBragg),0,np.sin(ThBragg)]
@@ -1019,13 +1096,12 @@ def raytrace(p,q,progress):
 
 ## POSITION OF THE SOURCE
         if (s['showrealspatial']):
-               if(random.random()<rOffsetRatio):
-                   r0=r0 + s['rOffset']
-                   spatialoffset=True
-                   effbeam=0
-               else:
-                   spatialoffset=False
-                   effbeam=1
+            spatialoffset=False
+            effbeam=1
+            if(random.random()<rOffsetRatio):
+                r0=r0 + s['rOffset']
+                spatialoffset=True
+                effbeam=0
         else:
             if (s['show2Dspatial']):
                 rr=random.random()
@@ -1052,32 +1128,32 @@ def raytrace(p,q,progress):
                 E0=Er
 #                E0=8980
                 contAddNow=1
-        else :
-            if (s['continuum']):
+        elif s['continuumMarks']:
+            if random.random()<0.0: #1000 eV broad continuum around centralRay
+                E0=random.random()*800-400 +s['EcentralRay']
+            else:
+                marknum=np.size(p['source']['continuumMarks_array'])
+                markind=np.random.randint(marknum)
+                E0=p['source']['continuumMarks_array'][markind]
+                contAddNow=1
+
+        elif (s['continuum']):
                 E0=np.random.rand()*EmaxBand+EcontAdd
                 contAddNow=1
 
-            else :
-                if s['continuumMarks']:
-                    if random.random()<0.0: #1000 eV broad continuum around centralRay
-                        E0=random.random()*800-400 +s['EcentralRay']
-                    else:
-                        marknum=s['continuumMarksCount']#number of marks to each side
-                        E0=s['EcentralRay']+(unidrnd(2*marknum+1)-marknum-1)*s['markSpread']
-                        contAddNow=1
-                else:        #default - mono or dichromatic
-                    if spatialoffset or (random.random()>s['secondBeamRatio']):
-                        E0=s['EcentralRay']#energy of the photon
-                    else:
-                        E0=s['EcentralRay']+s['secondBeamEOffset']
+        else:        #default - mono or dichromatic
+            if spatialoffset or (random.random()>s['secondBeamRatio']):
+                E0=s['EcentralRay']#energy of the photon
+            else:
+                E0=s['EcentralRay']+s['secondBeamEOffset']
 
-                    if (s['continuumAdd']) and (random.random()<s['continuumAddedRatio']):
-                        E0=random.random()*1000-500 +s['EcentralRay']
-                        contAddNow=1
-                        effbeam=0
-                    else:
-                        contAddNow=0
-                        effbeam=1
+            if (s['continuumAdd']) and (random.random()<s['continuumAddedRatio']):
+                E0=random.random()*1000-500 +s['EcentralRay']
+                contAddNow=1
+                effbeam=0
+            else:
+                contAddNow=0
+                effbeam=1
 
         if (effbeam):
             effcnt=effcnt+1
@@ -1404,7 +1480,7 @@ def spectrometer_evaluate(p,spectrorrr):
 
     q=ev['dispersionQuadratic'];
     ii=ii +'dispersion: E[eV] = ${:2.5f}d^2 + {:2.2f}d + {:2.0f} $'.format(q[0],q[1],q[2])+ nl
-
+    eV_per_px=q[1]*p['geometry']['detectorPxSize']
 ##################
     evaluateMono(p,mono_rrr)
 
@@ -1434,10 +1510,10 @@ def spectrometer_evaluate(p,spectrorrr):
     ii=ii+ 'vertical spread from fwhm:  {:2.3f}'.format(p['evalu']['verticalSpreadFWHM']) +' mm '+ nl
     ii=ii+ '  - energy resolution:  {:2.3f}'.format( EresolutionF) +' eV'+ nl
 
-
-    ii=ii+ 'vert. spr. narrow (fwhm) :  {:2.3f}'.format( p['evalu']['verticalSpreadFWHMNarrow']) +' mm '+ nl
-    ii=ii +'  - energy resolution:  {:2.3f}'.format( EresolutionFnarrow)+ ' eV' +nl
-
+    if p['geometry']['evaluation_width']>0:
+        ii=ii+ 'vert. spr. narrow (fwhm) :  {:2.3f}'.format( p['evalu']['verticalSpreadFWHMNarrow']) +' mm '+ nl
+        ii=ii +'  - energy resolution:  {:2.3f}'.format( EresolutionFnarrow)+ ' eV' +nl
+    ii=ii +'dispersion per px:  {:2.1f}'.format(eV_per_px)+ ' eV/px' +nl
 
     rOff=np.linalg.norm(so['rOffset'])
     SpectSpatialMagnification=p['evalu']['SpatialOffset']/rOff #- = mm/mm
@@ -1483,10 +1559,65 @@ def spectrometer_evaluate(p,spectrorrr):
     plt.savefig('mmpxrt_' +p['simulation']['name']+'.png' , bbox_inches ='tight',dpi=118)
     fn=p['simulation']['out_data_directory']  +'mmpxrt_parameters_' +p['simulation']['name']
     pickle.dump( p, open( fn, "wb" ) )
-    plt.close(fig)
+#    plt.close(fig)
     return fig, ax
 
 
+
+
+def spectrometer_evaluate_noncentral_PSFs(p,spectrorrr):
+    """
+    """
+    ####
+    so=p['source']
+    broad_rrr=spectrorrr['broad']
+
+    fig= plt.figure(figsize=(19.20,12.08))
+    ##################
+
+    E0s=broad_rrr['E0s']
+    rayres=broad_rrr['rayres']
+    marks=so['continuumMarks_array']
+    colores=broad_rrr['colores']
+    spectrorrr
+
+    yax=np.arange(-p['simulation']['PSFWindowY'],p['simulation']['PSFWindowY'],p['simulation']['PSFStepY'])
+    yax=yax[:-1]
+    profs=np.zeros((np.size(marks),np.size(yax)))
+
+    for mi,mark in enumerate(marks):
+        ax = fig.add_subplot(2,4,mi+1)
+        plt.title('{:.0f} eV'.format(mark))
+        sel=(E0s==mark)
+    #    sel=(E0s>0)
+        sel_rrr={}
+        #sel_rrr['effdet']=effdet
+        #sel_rrr['effcnt']=effcnt
+        sel_rrr['rayres']=rayres[sel,:,:]
+        #sel_rrr['aois']=aois
+        sel_rrr['E0s']=E0s[sel]
+        #sel_rrr['numrays']=numrays
+        sel_rrr['colores']=colores[sel]*0+1
+
+        [cntrsY,su2]=drawPSF(p,sel_rrr,ax,vertical_central=0)
+        profs[mi,:]=su2
+
+    # %
+    #fig.add_subplot(2,4,8)
+    plt.subplot2grid((2, 4), (1, 1), colspan=3)
+    for mi,mark in enumerate(marks):
+        prof=profs[mi,:]
+        prof=prof/np.max(prof)
+        plt.plot(yax*1e3,prof+mi*0.,label='{:.0f} eV'.format(mark))
+    plt.title('Vertical profile of PSFs')
+    plt.legend()
+    plt.ylim(0,1)
+    plt.xlabel('relative position [μm]')
+    plt.grid()
+
+    plt.savefig('mmpxrt_' +p['simulation']['name']+'noncentral_PSFs.png' , bbox_inches ='tight',dpi=118)
+    return fig, ax
+
 def evaluateCont(p,rrr):
     ## generationg pos* and calculation Erangemax
     sg=p['sg']
@@ -1681,25 +1812,31 @@ def evaluateMono(p,rrr):
         p['evalu']['PSF_horizontalSpreadRMS']=0
         return
 
-    rang=(np.nanmax(vals)-np.nanmin(vals))
-    lene=np.size(vals)-np.sum(np.isnan(vals))
-    Ebin=rang/lene*80
-    #if PSFStepY>-1:
-        #Ebin=PSFStepY
-    Ex=np.arange(np.nanmin(vals)-Ebin,rang+np.nanmax(vals),Ebin)
-    h=np.histogram(vals,Ex)[0]
-
-    hm=np.max(h)/2
-    dat2=h
-    dat2[dat2<=hm]=0
-    dat2[dat2>hm]=1
-    if (False):
-        plt.plot(Ex,h)
-        plt.plot(Ex,dat2*hm)
-    Ex2=Ex*np.append(dat2,0)
-    Ex2[Ex2==0]=np.nan
-    verticalSpreadFWHM=np.nanmax(Ex2)-np.nanmin(Ex2)
+    if 0: #outdated,  done in drawPSF
+        rang=(np.nanmax(vals)-np.nanmin(vals))
+        lene=np.size(vals)-np.sum(np.isnan(vals))
+        Ebin=rang/lene*380
+        #if PSFStepY>-1:
+            #Ebin=PSFStepY
+        print(Ebin)
+        Ex=np.arange(np.nanmin(vals)-Ebin,rang+np.nanmax(vals),Ebin)
+        h=np.histogram(vals,Ex)[0]
 
+        hm=np.max(h)/2
+        dat2=h
+        dat2[dat2<=hm]=0
+        dat2[dat2>hm]=1
+        Ex2=Ex*np.append(dat2,0)
+        Ex2[Ex2==0]=np.nan
+        if (True):
+            h2=np.append(h,0)
+            plt.figure()
+            plt.plot(Ex,h2)
+            plt.plot(Ex2,np.append(dat2,0)*hm)
+            plt.figure()
+        verticalSpreadFWHM=np.nanmax(Ex2)-np.nanmin(Ex2)
+    else:
+        verticalSpreadFWHM=0
 
     ## Horizontal FWHM through histogram
     vals=posR[:,0]
@@ -1725,41 +1862,42 @@ def evaluateMono(p,rrr):
 
     ## Vertical FWHM through histogram- narrow Y range
     evalSelectY=p['geometry']['evaluation_width']
-    aa=np.array(posR[:,0])
-    selnonnan=(~np.isnan(aa));
-    selnonnan[selnonnan]=(np.abs(posR[selnonnan,0])<evalSelectY/2)
-    #selnonnan=selnonnan*0+1
-    sel=selnonnan
-    #sel=np.logical_and((np.abs(posR[:,0])<evalSelectY/2))
+    verticalSpreadFWHMnarrow=np.nan
+    verticalSelectPSF=np.nan
 
-    vals=posR[sel,1]
-    rangee=(np.nanmax(vals)-np.nanmin(vals))
-    lena=np.size(vals)-np.sum(np.isnan(vals))
-    Ebin=rangee/lena*80
+    if evalSelectY>0:
+        aa=np.array(posR[:,0])
+        selnonnan=(~np.isnan(aa));
+        selnonnan[selnonnan]=(np.abs(posR[selnonnan,0])<evalSelectY/2)
+        #selnonnan=selnonnan*0+1
+        sel=selnonnan
+        #sel=np.logical_and((np.abs(posR[:,0])<evalSelectY/2))
 
-    Ebin=max(p['geometry']['detectorPxSize'],Ebin)
+        vals=posR[sel,1]
+        rangee=(np.nanmax(vals)-np.nanmin(vals))
+        lena=np.size(vals)-np.sum(np.isnan(vals))
+        Ebin=rangee/lena*80
 
+        Ebin=max(p['geometry']['detectorPxSize'],Ebin)
 
-    #if PSFStepY>-1:
-        #Ebin=PSFStepY
-    if np.size(vals)>1:
-        Ex=np.arange(np.nanmin(vals)-Ebin,rangee+np.nanmax(vals),Ebin)
-        h=np.histogram(vals,Ex)[0]
+        #if PSFStepY>-1:
+            #Ebin=PSFStepY
+        if np.size(vals)>1:
+            Ex=np.arange(np.nanmin(vals)-Ebin,rangee+np.nanmax(vals),Ebin)
+            h=np.histogram(vals,Ex)[0]
+
+            hm=np.max(h)/2
+            dat2=np.copy(h)
+            dat2[dat2<=hm]=0
+            dat2[dat2>hm]=1
+            if (False):
+                plt.plot(Ex,h,'k')
+                plt.plot(Ex,dat2*hm,'k')
+            Ex2=Ex*np.append(dat2,0)
+            Ex2[Ex2==0]=np.nan
+            verticalSelectPSF=np.matrix((Ex,np.append(np.nan,h)))
+            verticalSpreadFWHMnarrow=np.nanmax(Ex2)-np.nanmin(Ex2)
 
-        hm=np.max(h)/2
-        dat2=np.copy(h)
-        dat2[dat2<=hm]=0
-        dat2[dat2>hm]=1
-        if (False):
-            plt.plot(Ex,h,'k')
-            plt.plot(Ex,dat2*hm,'k')
-        Ex2=Ex*np.append(dat2,0)
-        Ex2[Ex2==0]=np.nan
-        verticalSelectPSF=np.matrix((Ex,np.append(np.nan,h)))
-        verticalSpreadFWHMnarrow=np.nanmax(Ex2)-np.nanmin(Ex2)
-    else:
-        verticalSpreadFWHMnarrow=np.nan
-        verticalSelectPSF=np.nan
     ## source size broadening (aka 2D resolution)
     if np.logical_and(np.nansum(np.abs([posB[:,1]]))>0, np.nansum(np.abs([posR[:,1]]))>0):
         meanOffy=np.nanmean([posB[:,1]])
@@ -1783,8 +1921,14 @@ def evaluateMono(p,rrr):
 
         #those 3 rows are the equaiton in paper.
         coveredratio=coveredangle/(4*np.pi)
+
         efficiency=reflectedratio*coveredratio
         efficiency=efficiency*c['crystalPeakReflectivity']
+        debug=1
+        if debug:
+            print("covered ratio: {:.1e}".format(coveredratio))
+            print("reflected ratio: {:.1e}".format(reflectedratio))
+            print("efficiency: {:.1e}".format(efficiency))
     else:
         efficiency=-1
 
@@ -1805,7 +1949,7 @@ def evaluateMono(p,rrr):
 
 
 ################################################################
-def drawPSF(p,rrr,ax):
+def drawPSF(p,rrr,ax,vertical_central=1):
     colores=rrr['colores']
     rayres=rrr['rayres']
     E0s=rrr['E0s']
@@ -1874,14 +2018,21 @@ def drawPSF(p,rrr,ax):
         windowY=p['simulation']['PSFWindowY']
     if p['simulation']['PSFStepY']>-1:
         stepY=p['simulation']['PSFStepY']
+    if p['simulation']['PSFStepX']>-1:
+        stepX=p['simulation']['PSFStepX']
     if p['simulation']['PSFWindowX']>-1:
         windowX=p['simulation']['PSFWindowX']
 
     if windowY<=(stepY*4):
         windowY=stepY*5
     ## plt.plotting
-    cntrsX=np.arange(-1*windowX,windowX,stepX)
+    if vertical_central:
+        vertical_mean=0
+    else:
+        vertical_mean=np.nanmean(pC[:,1])
     cntrsY=np.arange(-1*windowY,windowY,stepY)
+    cntrsY=cntrsY+vertical_mean
+    cntrsX=np.arange(-1*windowX,windowX,stepX)
 
     edX=cntrsX+stepX/2
     edY=cntrsY+stepY/2
@@ -1893,10 +2044,9 @@ def drawPSF(p,rrr,ax):
     nR[0,np.shape(nR)[1]-1]=0
     nR[np.shape(nR)[0]-1,0]=0
     nR2=np.transpose(nR)
-    plt.imshow(nR2,extent=(-1000*windowX,1000*windowX,-1000*windowY,1000*windowY),aspect='auto',origin='lower')
-
-    ## profiles
+    plt.imshow(nR2,extent=(-1000*windowX,1000*windowX,1000*(-windowY+vertical_mean),1000*(windowY+vertical_mean)),aspect='auto',origin='lower')
 
+# %% get profiles & FWHM
     su2=np.sum(nR,0)
     su2y=np.sum(nR,1)
     su2[0]=0
@@ -1904,31 +2054,61 @@ def drawPSF(p,rrr,ax):
     su2y[0]=0
     su2y[np.size(su2y)-1]=0
 
-    su2=su2/np.max(su2)*0.5*windowX-windowX
-    su2y=su2y/np.max(su2y)*0.5*windowY-windowY
 
-    plt.plot(np.append(np.nan,su2)*1e3,cntrsY*1000,'w',linewidth=1)
-    plt.plot(cntrsX*1e3,np.append(np.nan,su2y*1e3),'w',linewidth=1)
+    profile=np.append(np.nan,su2)
+    hm=np.nanmax(profile)/2
+    dat2=profile*1.0
+    dat2[dat2<=hm]=np.nan
+    dat2[dat2>hm]=1
+#    Ex2=cntrsY*np.append(dat2,0)
+    if 0:
+        plt.figure()
+        plt.plot(cntrsY,profile)
+        plt.plot(cntrsY,dat2*hm)
+#        plt.plot(Ex2,np.append(dat2,0)*hm)
+        plt.figure()
+    verticalSpreadFWHM=(np.nansum(dat2)+0.5)*stepY
+    p['evalu']['verticalSpreadFWHM']=verticalSpreadFWHM
+    #print(verticalSpreadFWHM)
+
+# %%  draw profiles
+
+    su2m=su2/np.max(su2)*0.5*windowX-windowX
+    su2y=su2y/np.max(su2y)*0.5*windowY-windowY+vertical_mean
+    print(windowY)
+    dat3=dat2*0.25*windowX-windowX
+    print(dat3)
+
+    plt.plot(np.append(np.nan,su2m)*1e3,(cntrsY)*1000,'w',linewidth=1) #vertical
+    plt.plot(dat3*1e3,cntrsY*1e3,'w',linewidth=1)
+
+    plt.plot(cntrsX*1e3,np.append(np.nan,(su2y)*1e3),'w',linewidth=1) #horizontal
+
+
+
+# %%
+
 
     #selected profile
     evalSelectY=p['geometry']['evaluation_width']
-    es2=evalSelectY/2*1e3
-    wy=windowY*1e3*1.1
-    plt.plot([es2,es2,-1*es2,-1*es2],[wy,-1*wy,-1*wy,wy],'r',linewidth=1)
+    if evalSelectY>0:
+        es2=evalSelectY/2*1e3
+        wy=windowY*1e3*1.1
 
-    #print(p['evalu']['verticalSelectPSF'])
-    if np.size(p['evalu']['verticalSelectPSF'])>1:
-        vyy=np.squeeze(np.asarray(p['evalu']['verticalSelectPSF'][1,:]))
-        vyy=(vyy/np.nanmax(vyy)*0.5*windowX-windowX)*1e3
-        plt.plot(vyy,np.squeeze(np.asarray(p['evalu']['verticalSelectPSF'][0,:]))*1e3,'r',linewidth=1)
+        if np.size(p['evalu']['verticalSelectPSF'])>1:
+            vyy=np.squeeze(np.asarray(p['evalu']['verticalSelectPSF'][1,:]))
+            vyy=(vyy/np.nanmax(vyy)*0.5*windowX-windowX)*1e3
+            plt.plot(vyy,np.squeeze(np.asarray(p['evalu']['verticalSelectPSF'][0,:]))*1e3,'r',linewidth=1)
+            plt.plot([es2,es2,-1*es2,-1*es2],[wy,-1*wy,-1*wy,wy],'r',linewidth=1)
 
-    plt.xlabel('y [$\mathrm{{\mu}m}$]')
-    plt.ylabel('d [$\mathrm{{\mu}m}$]')
-    ax.yaxis.set_label_coords(-0.12,0.5)
-    #colorbar
-    plt.xlim(-1*windowX*1e3,windowX*1e3)
-    plt.ylim(-1*windowY*1e3,windowY*1e3)
+        plt.xlabel('y [$\mathrm{{\mu}m}$]')
+        plt.ylabel('d [$\mathrm{{\mu}m}$]')
+        ax.yaxis.set_label_coords(-0.12,0.5)
+        #colorbar
+        plt.xlim(-1*windowX*1e3,windowX*1e3)
+        plt.ylim(-1*windowY*1e3,windowY*1e3)
 
+    return [cntrsY,su2]
 
 def drawDetectorStripes(p,rrr,ax,realscale):
     colores=rrr['colores']
@@ -1968,6 +2148,12 @@ def drawDetectorStripes(p,rrr,ax,realscale):
     stepY=min(windowY/3,stepY)
     stepX=min(windowX/3,stepX)
 
+    if p['simulation']['DetectorStepY']>-1:
+        stepY=p['simulation']['DetectorStepY']
+    if p['simulation']['DetectorStepX']>-1:
+        stepX=p['simulation']['DetectorStepX']
+
+
     cntrsX=np.arange(-1*windowX,windowX,stepX)
     cntrsY=np.arange(-1*windowY,windowY,stepY)
 
@@ -2020,7 +2206,10 @@ def drawDetectorStripes(p,rrr,ax,realscale):
     plt.plot([0, 0], [-600, 600],'-',linewidth=1,color=[0.5 ,0.5 ,0.5])
     plt.plot([-1, 1 ], [0, 0],'-',linewidth=1,color=[0.5 ,0.5, 0.5])
 
-    plt.xlim(-1*windowX,windowX)
+    if np.size(p['simulation']['DetectorXlim'])>0:
+        plt.xlim(p['simulation']['DetectorXlim'])
+    else:
+        plt.xlim(-1*windowX,windowX)
     plt.ylim(-1*windowY,windowY)
 
     if not realscale:
@@ -2044,6 +2233,104 @@ def drawDetectorStripes(p,rrr,ax,realscale):
         envel=np.matrix((cntrsY,envelope))
         p['evalu']['envelope']=envel
 
+
+def drawDetectorAll(p,rrr,ax,realscale):
+    colores=rrr['colores']
+    rayres=rrr['rayres']
+    E0s=rrr['E0s']
+    Sdetector=p['sg']['Sdetector']
+
+    numrays=np.size(colores)
+    posCont=np.full((numrays,2),np.nan)
+    posR=np.full((numrays,2),np.nan)
+    posG=np.full((numrays,2),np.nan)
+    posB=np.full((numrays,2),np.nan)
+    posCont=np.full((numrays,2),np.nan)
+    posAll=np.full((numrays,2),np.nan)
+    po=np.array([0.,0.],float)
+    detn=p['sg']['detn']
+    detHorz=(np.abs(detn[2])>np.abs(detn[0]))
+    for i in np.arange(np.shape(rayres)[0]):
+          absPos=rayres[i,2,:]
+       #projection
+          po[0]=absPos[1]#y in 3D becomes x in 2D
+          po[1]=((absPos[0]-Sdetector[0])**2 + (absPos[2]-Sdetector[2])**2) ** (1/2)#distance from detector center
+#          posAll[i,:]=[absPos[1], absPos[0]]
+          if detHorz:
+              if (absPos[0]-Sdetector[0])<0:#see if it was 'front' or 'behind' the detector
+                  po[1]=po[1]*-1
+          else:
+              if (absPos[2]-Sdetector[2])<0:
+                  po[1]=po[1]*-1
+          posAll[i,:]=po
+          if colores[i]==1:
+              posR[i,:]=po
+          else:
+              if colores[i]==2:
+                  posG[i,:]=po
+              else:
+                  if colores[i]==3:
+                      posB[i,:]=po
+          if colores[i]==4:
+            posCont[i,:]=po
+
+#    if (np.nansum(np.nansum(posR))==0):
+ #       return
+    ## get the optimal ranges
+    pC=posAll
+    xs=np.abs(pC[:,0])
+    ys=np.abs(pC[:,1])
+
+    stepX=p['geometry']['detectorPxSize']
+    stepY=p['geometry']['detectorPxSize']
+    windowY=p['geometry']['detectorWidth']
+    windowX=p['geometry']['detectorLength']
+
+    ## plt.plotting
+    cntrsX=np.arange(-1*windowX,windowX,stepX)
+    cntrsY=np.arange(-1*windowY,windowY,stepY)
+
+    edX=cntrsX+stepX/2
+    edY=cntrsY+stepY/2
+    #cntrs=[cntrsX,cntrsY]
+    eds=[edX,edY]
+    nC = np.histogram2d(pC[:,0],pC[:,1],eds)[0]
+    nC[0,0]=0
+    nC[np.shape(nC)[0]-1,np.shape(nC)[1]-1]=0
+    nC[0,np.shape(nC)[1]-1]=0
+    nC[np.shape(nC)[0]-1,0]=0
+    nR2=np.transpose(nC)
+    plt.imshow(nR2,extent=(-1000*windowX,1000*windowX,-1000*windowY,1000*windowY),aspect='auto',origin='lower')
+
+    ## profiles
+
+    su2=np.sum(nC,0)
+    su2y=np.sum(nC,1)
+    su2[0]=0
+    su2[np.size(su2)-1]=0
+    su2y[0]=0
+    su2y[np.size(su2y)-1]=0
+
+    su2=su2/np.max(su2)*0.5*windowX-windowX
+    su2y=su2y/np.max(su2y)*0.5*windowY-windowY
+
+    plt.plot(np.append(np.nan,su2)*1e3,cntrsY*1000,'w',linewidth=1)
+    plt.plot(cntrsX*1e3,np.append(np.nan,su2y*1e3),'w',linewidth=1)
+
+    #selected profile
+    evalSelectY=p['geometry']['evaluation_width']
+    es2=evalSelectY/2*1e3
+    wy=windowY*1e3*1.1
+    plt.plot([es2,es2,-1*es2,-1*es2],[wy,-1*wy,-1*wy,wy],'r',linewidth=1)
+
+    plt.xlabel('y [$\mathrm{{\mu}m}$]')
+    plt.ylabel('d [$\mathrm{{\mu}m}$]')
+    ax.yaxis.set_label_coords(-0.12,0.5)
+    #colorbar
+    plt.xlim(-1*windowX*1e3,windowX*1e3)
+    plt.ylim(-1*windowY*1e3,windowY*1e3)
+
+
 def showDispersion(p,rrr,ax,plotnow):
     from numpy.polynomial import polynomial as P
     dispersionVertical=0
@@ -2121,22 +2408,22 @@ def plotoneview(p,rrr,maxrays,ax):
     for i in np.arange(maxrays):
         c=colores[i];
         if (c==1):
-            col='r-';
-            co='r*';
-            co2=[0.7, 0.3,0.2];
+            col='-'
+            co='*'
+            co2=[0.7, 0.3,0.2]
         else  :
             if (c==2):
-                col='g-';
-                co='g*';
-                co2=[0.2, 0.7,0.1];
+                col='-'
+                co='*'
+                co2=[0.2, 0.7,0.1]
             else :
-             col='b-';
-             co='b.';
-             co2=[0.3, 0.6,0.99];
+             col='-'
+             co='.'
+             co2=[0.3, 0.6,0.99]
 
-        plt.plot(rayres[i,:,0],rayres[i,:,1],rayres[i,:,2],col,linewidth=lw,color=co2);
-        plt.plot([rayres[i,2,0]],[rayres[i,2,1]],[rayres[i,2,2]],co,linewidth=lw,color=co2);
-        plt.plot([rayres[i,1,0]],[rayres[i,1,1]],[rayres[i,1,2]],co,linewidth=lw,color=co2);
+        plt.plot(rayres[i,:,0],rayres[i,:,1],rayres[i,:,2],col,linewidth=lw,color=co2)
+        plt.plot([rayres[i,2,0]],[rayres[i,2,1]],[rayres[i,2,2]],co,linewidth=lw,color=co2)
+        plt.plot([rayres[i,1,0]],[rayres[i,1,1]],[rayres[i,1,2]],co,linewidth=lw,color=co2)
 #    %% lost beams
 #    maxrays=numrays;
 #    maxrays=min(300,maxrays);
@@ -2189,7 +2476,7 @@ def drawCrystal(p):
     crystalMaxX=crystalCenterX+c['length']/2
     crystalR=c['radius_w']
     crystalRv=c['radius_l']
-    xis=np.arange(crystalMinX,crystalMaxX,step=step)
+    xis=np.arange(crystalMinX,crystalMaxX, step=step)
     zs=np.zeros(np.size(xis))
     zs2=np.zeros(np.size(xis))
 
@@ -2210,7 +2497,7 @@ def drawCrystal(p):
     plt.plot(xis,xis*0-cc,zs,'k-')
     plt.plot(xis,xis*0,zs2,'k-')
 
-    yis=np.arange(-1*cc,cc,step=step)
+    yis=np.arange(-1*cc,cc+1, step=step)
     zs=np.zeros(np.size(yis))
     zs2=np.zeros(np.size(yis))
     zs3=np.zeros(np.size(yis))
@@ -2233,10 +2520,7 @@ def drawCrystal(p):
 
     plt.plot(yis*0+crystalMinX,yis,zs,'k-')
     plt.plot(yis*0+crystalMaxX,yis,zs2,'k-')
-    plt.plot(yis*0+crystalCenterX,yis,zs3,'k-')
-    print(yis)
-    print(zs2)
-
+    #plt.plot(yis*0+crystalCenterX,yis,zs3,'k-')
 
 def drawSideView(p,broad_rrr,ax):
     oneViewRays=50
@@ -2250,22 +2534,22 @@ def drawSideView(p,broad_rrr,ax):
     for i in np.arange(maxrays):
         c=colores[i];
         if (c==1):
-            col='r-';
-            co='r*';
+            col='-';
+            co='*';
             co2=[0.7, 0.3,0.2];
         else  :
             if (c==2):
-                col='g-';
-                co='g*';
+                col='-';
+                co='*';
                 co2=[0.2, 0.7,0.1];
             else :
-             col='b-';
-             co='b.';
+             col='-';
+             co='.';
              co2=[0.3, 0.6,0.99];
 
-        plt.plot(rayres[i,:,0],rayres[i,:,2],col,linewidth=1,color=co2);
-        plt.plot([rayres[i,2,0]],[rayres[i,2,2]],co,linewidth=1,color=co2);
-        plt.plot([rayres[i,1,0]],[rayres[i,1,2]],co,linewidth=1,color=co2);
+        plt.plot(rayres[i,:,0],rayres[i,:,2],col,linewidth=1,color=co2)
+        plt.plot([rayres[i,2,0]],[rayres[i,2,2]],co,linewidth=1,color=co2)
+        plt.plot([rayres[i,1,0]],[rayres[i,1,2]],co,linewidth=1,color=co2)
 
 #drawCrystal(p)
     c=p['crystal']
